
/* Network traffic analyzer
 * (c) 2001-2002, Emil Mikulic.
 */

/* Port database */
#include "port_db.h"



/* Global database */
db port_db;



void port_db_init(void)
{
	db_init(&port_db);
}

void port_db_free(void)
{
	unsigned int i;

	for (i=0; i<port_db.used; i++)
		free(port_db.records[i]);

	db_free(&port_db);
}



/* port_record needs to be deallocated when it's no longer needed */
inline port_record *port_make(const word port)
{
	port_record *p = (port_record*)malloc(sizeof(port_record));

	p->port = port;
	SET64(p->data_in, 0,0);
	SET64(p->data_out, 0,0);

	return p;
}



/* update transfer amount */
inline void port_update_in(port_record *p, const dword in)
{
	add_64bit(p->data_in, in);
}

inline void port_update_out(port_record *p, const dword out)
{
	add_64bit(p->data_out, out);
}



/* find a port */
inline port_record *port_find(const word port)
{
	int i;

	if (port_db.used == 0)
		return NULL;

	for (i=0; i<port_db.used; i++)
	{
		port_record *tmp = (port_record*)db_get(port_db,i);
		assert(tmp != NULL);
		if (tmp->port == port) return tmp;
	}

	return NULL;
}



/* add or update a port, depending on whether it exists in the db or not */
inline port_record *port_from_num(const word port)
{
	port_record *p;

	p = port_find(port);

	if (p == NULL)
	{
		p = port_make(port);
		if (!db_add(&port_db, p)) freakout("couldn't add to port_db in port_from_num()");
	}

	return p;
}



/* sorting juju */
void port_qs_swap(port_record **list, i64 *amount, int a, int b)
{
	i64 t64;
	port_record *tpr;

/*	assert(a >= 0);
	assert(b >= 0);
	assert(a < port_db.used);
	assert(b < port_db.used); */

	t64 = amount[a];
	amount[a] = amount[b];
	amount[b] = t64;

	tpr = list[a];
	list[a] = list[b];
	list[b] = tpr;
}



void port_quicksort(port_record **list, i64 *amount, int left, int right)
{
	int i, j;
	i64 v;

	if(right > left) {
		v = amount[right];
		i = left - 1;
		j = right;
		for(;;) {
			do {
				i++;
				if (i >= j) break;
			} while (smaller_64(amount[i], v));

			do {
				j--;
				if (i >= j) break;
			} while (bigger_64(amount[j], v));

			if (i >= j) break;
			port_qs_swap(list, amount, i, j);
		}
		port_qs_swap(list, amount, i, right);
		port_quicksort(list, amount, left, i - 1);
		port_quicksort(list, amount, i + 1, right);
	}
}

/* note: it'd be a really good idea to lock the port_db mutex before doing this */
inline port_record **port_db_sort(sort_type st)
{
	port_record **list;
	i64 *amount;
	int i;

	list = (port_record**)malloc(port_db.used * sizeof(port_record*));
	amount = (i64*)malloc(port_db.used * sizeof(i64));

	for (i=0; i<port_db.used; i++)
	{
		/* initialize port record */
		list[i] = (port_record*)db_get(port_db, i);

		/* initialize amount */
		switch(st)
		{
case TOTAL:
#ifdef HAVE_QUAD
	amount[i] = list[i]->data_in + list[i]->data_out;
#else
	amount[i] = list[i]->data_in;                   /* total = in */
	add_64bit(amount[i], list[i]->data_out.lo);    /* plus out.lo */
	amount[i].hi += list[i]->data_out.hi;           /* plus out.hi */
#endif
	break;
case IN:
	amount[i] = list[i]->data_in;
	break;
case OUT:
	amount[i] = list[i]->data_out;
	break;
default:
	freakout("invalid sort_type passed to port_db_sort()");
		}
	}

	port_quicksort(list, amount, 0, port_db.used-1);

	free(amount);
	return list;
}



void port_db_dump(FILE *fp)
{
	int i;

	fprintf(fp, "%u\n", port_db.used);
	for (i=0; i<port_db.used; i++)
	{
		port_record *p = (port_record*)db_get(port_db, i);

		fprintf(fp, "%u|%u:%u|%u:%u\n",
				p->port,
				HI(p->data_in),
				LO(p->data_in),
				HI(p->data_out),
				LO(p->data_out));
	}
}

